import React from 'react';
import { Formik, FieldArray } from 'formik';

import classNames from 'classnames';

import { Col, Row, UncontrolledTooltip } from 'reactstrap';

import * as _ from 'lodash';

import { FiPlusCircle, FiMinusCircle } from 'react-icons/fi';

import RequiredField from 'components/lib/forms/RequiredField';

import Form from 'models/Form';
import FieldConfig from 'models/InputFieldConfig';

import { DisplayFormikState } from './FormikDebug';
import { PromptUnsavedForm } from './PromptUnsavedForm';

import Button from './Button';
import Checkbox from './Checkbox';
import CheckboxGroup from './CheckboxGroup';
import CustomDatePicker from './DatePicker';
import InputText from './InputText';
import InputToggle from './InputToggle';
// import InputDecimal from './InputDecimal';
import InputEmail from './InputEmail';
import InputNumber from './InputNumber';
import Password from './Password';
import RadioButton from './RadioButton';
import ReactSelect from './Select';

interface Props {
  key: string;
  form: Form;
  fields: Array<FieldConfig>;
  errors?: Array<any>;
  layout: 'grid' | 'vertical' | 'horizontal' | 'custom';
  layoutGrid?: number;
  validation?: any;
  onFormSubmit: Function;
  labelPosition?: 'left' | 'right';
  changeHandler?: Function;
  promptIfUnsaved?: true | false;
}

class DynamicForm extends React.PureComponent<Props, any> {
  static defaultProps = {
    layout: 'vertical',
    layoutGrid: 12,
    labelPosition: 'left',
    key: '',
  };

  componentWillMount() {
    this.setState({
      layout: this.props.layout,
      layoutGrid: this.props.layoutGrid,
    });
  }

  renderField(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    if (input.type === 'multi_select' || input.type === 'select') {
      return this.renderSelect(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'checkbox_group') {
      return this.renderCheckboxes(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'checkbox') {
      return this.renderCheckbox(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'number') {
      return this.renderInputNumber(name, input, form, classes, lClass, dClass);
    }

    /*
    if (input.type === 'textarea') {
      return this.renderTextarea(input);
    }
    */

    if (input.type === 'password') {
      return this.renderPassword(name, input, form, classes, lClass, dClass);
    }

    /*
    if (input.type === 'timepicker') {
      return this.renderTimePicker(input);
    }
    */

    if (input.type === 'datepicker') {
      return this.renderDatePicker(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'text') {
      return this.renderTextbox(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'email') {
      return this.renderEmail(name, input, form, classes, lClass, dClass);
    }
    if (input.type === 'radio') {
      return this.renderRadioButtons(
        name,
        input,
        form,
        classes,
        lClass,
        dClass
      );
    }
    if (input.type === 'toggle') {
      return this.renderToggle(name, input, form, classes, lClass, dClass);
    }

    /*
    if (input.type === 'decimal') {
      return this.renderDecimal(input, form, classes, lClass, dClass);
    }
    */

    if (input.type === 'array_fields') {
      return this.renderArray(input, form, classes);
    }
    if (input.type === 'button') {
      return this.renderButton(
        name,
        input,
        form,
        input.classNames,
        lClass,
        dClass
      );
    }
  }

  renderFields(
    inputs: Array<FieldConfig>,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return inputs.map(input => {
      return this.renderField(input.name, input, form, classes, lClass, dClass);
    });
  }

  renderArray(input: FieldConfig, form: any, classes: any): any {
    const o = {};

    _.map(input.arrayFields.headers, function(i) {
      return (o[i.key] = '');
    });

    let cn: any;

    if (this.props.layout === 'grid') {
      cn = classNames({
        'col-md-12': true,
      });
      classes = 'col-md-12 no-gutters';
    } else if (this.props.layout === 'horizontal') {
      const rg = 12 - this.props.layoutGrid;

      cn = classNames({
        [`offset-md-${this.props.layoutGrid}`]: true,
        [`col-md-${rg}`]: true,
      });
    }

    return (
      <FieldArray
        key={input.name}
        name={input.name}
        render={arrayHelpers => (
          <div className={classes}>
            <div className={cn}>
              <table className="table table-bordered">
                <thead>
                  <tr>
                    {input.arrayFields.headers.map((i: any, index: number) => (
                      <th key={`${i.id}+${index}`}>
                        {i.label} {i.required && <RequiredField />}
                      </th>
                    ))}
                    <th>Actions</th>
                  </tr>
                </thead>
                <tbody>
                  {form.values[input.name].map((f: any, rowIndex: number) => {
                    return (
                      <tr key={`${f.name}+${rowIndex}`}>
                        {input.arrayFields.fields.map(
                          (i: any, index: number) => {
                            i.name = `${input.name}.${rowIndex}.${i.id}`;
                            return (
                              <td key={`{i.name}+${index}`}>
                                {this.renderField(i.name, i, form, '', '', '')}
                              </td>
                            );
                          }
                        )}
                        <td>
                          <div className="btn-group">
                            <span
                              onClick={() => arrayHelpers.push(o)}
                              id={`tt-add-${input.id}-${rowIndex}`}
                            >
                              <FiPlusCircle size={18} />
                            </span>
                            <UncontrolledTooltip
                              placement="top"
                              target={`tt-add-${input.id}-${rowIndex}`}
                            >
                              Add
                            </UncontrolledTooltip>
                            &nbsp;
                            <span
                              onClick={() => {
                                return rowIndex > 0
                                  ? arrayHelpers.remove(rowIndex)
                                  : null;
                              }}
                              id={`tt-remove-${input.id}-${rowIndex}`}
                            >
                              <FiMinusCircle size={18} />
                            </span>
                            <UncontrolledTooltip
                              placement="top"
                              target={`tt-remove-${input.id}-${rowIndex}`}
                            >
                              Remove
                            </UncontrolledTooltip>
                          </div>
                        </td>
                      </tr>
                    );
                  })}
                </tbody>
              </table>
            </div>
          </div>
        )}
      />
    );
  }

  /*
  renderDecimal(
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return <InputDecimal field={input} form={form} classes={classes} lClass={lClass}
    dClass={dClass} />;
  }
  */

  renderInputNumber(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <InputNumber
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderPassword(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <Password
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderToggle(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <InputToggle
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderTextbox(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    // input.arrayIndex = index;
    return (
      <InputText
        key={name}
        field={input}
        form={form}
        name={name}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
        // onEvent={this.props.changeHandler}
      />
    );
  }

  renderEmail(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <InputEmail
        key={name}
        field={input}
        form={form}
        name={name}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderCheckbox(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <Checkbox
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderCheckboxes(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <CheckboxGroup
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderRadioButtons(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <RadioButton
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  renderButton(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string,
    buttonType?: 'button' | 'submit' | 'reset'
  ) {
    return (
      <Button
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
        buttonType={buttonType}
      />
    );
  }

  renderDatePicker(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <CustomDatePicker
        name={name}
        key={name}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }

  /*
  renderFile(input: FieldConfig) {}

  renderTextarea(input: FieldConfig) {}

  renderTimePicker(input: FieldConfig) {}

  renderDateTimePicker(
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    return (
      <CustomDatePicker

        key={input.id}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
      />
    );
  }
  */

  renderSelect(
    name: string,
    input: FieldConfig,
    form: any,
    classes: any,
    lClass: string,
    dClass: string
  ) {
    // input.arrayIndex = index;
    return (
      <ReactSelect
        name={name}
        key={name}
        value={form.values[input.name]}
        field={input}
        form={form}
        classes={classes}
        lClass={lClass}
        dClass={dClass}
        isMulti={input.isMulti}
        onChange={form.setFieldValue}
        onBlur={form.setFieldTouched}
        onEvent={this.props.changeHandler}
      />
    );
  }

  getInitialValues(inputs: any) {
    const initialValues: any = {};

    inputs.forEach((field: any) => {
      if (!initialValues[field.name]) {
        initialValues[field.name] = field.value;
      }
    });

    return initialValues;
  }

  render() {
    const initialValues = this.getInitialValues(this.props.fields);

    let formContainerClass = classNames({
      row: this.props.layout === 'grid',
      right: this.props.labelPosition === 'right',
    });

    let cn: any;

    if (this.props.layout === 'grid') {
      cn = classNames({
        'form-group': true,
        [`col-md-${this.props.layoutGrid}`]: true,
      });
    } else if (this.props.layout === 'vertical') {
      cn = classNames({
        'form-group': true,
      });
    } else if (this.props.layout === 'horizontal') {
      cn = classNames({
        'form-group': true,
        row: true,
      });
    }

    let lClass: string = '';
    let dClass: string = '';
    let bClass: string = '';
    let bCnClass: string = cn;

    if (this.props.layout === 'horizontal') {
      lClass = classNames({
        [`col-md-${this.props.layoutGrid}`]: true,
      });

      const rg = 12 - this.props.layoutGrid;

      dClass = classNames({
        [`col-md-${rg}`]: true,
      });

      formContainerClass = classNames({
        horizontal: true,
        right: this.props.labelPosition === 'right',
      });

      bClass = classNames({
        [`offset-md-${this.props.layoutGrid}`]: true,
      });
    }

    if (this.props.layout === 'grid') {
      bCnClass = classNames({
        'form-group': true,
      });
    }

    bClass = classNames(bClass, {
      'text-right': this.props.form.buttonPosition === 'right',
    });

    return (
      <Formik
        key={this.props.key}
        enableReinitialize={true}
        onSubmit={values => {
          this.props.onFormSubmit(values);
        }}
        validationSchema={this.props.validation}
        initialValues={initialValues}
      >
        {formik => (
          <div>
            <Row>
              <Col sm={{ size: 12 }}>
                <div>
                  <form onSubmit={formik.handleSubmit}>
                    {this.props.promptIfUnsaved ? <PromptUnsavedForm /> : ''}
                    <div className={formContainerClass}>
                      {this.renderFields(
                        this.props.fields,
                        formik,
                        cn,
                        lClass,
                        dClass
                      )}
                    </div>
                    <div className={bCnClass}>
                      <div className={`${bClass} ${dClass}`}>
                        {this.renderFields(
                          this.props.form.buttons,
                          formik,
                          cn,
                          lClass,
                          dClass
                        )}
                      </div>
                    </div>
                  </form>
                </div>
              </Col>
            </Row>
            {this.props.form.isDebug && (
              <Row>
                <Col sm={{ size: 12 }}>
                  <DisplayFormikState {...formik} />
                </Col>
              </Row>
            )}
          </div>
        )}
      </Formik>
    );
  }
}

export default DynamicForm;
